

@=========================  PROCEDURES ===================@
@=========================================================@
/*
*  {P,VC_U, VC_pv, pvec, std_pvec, t_pvec} = sure(Y,X0);
** simple procedure to estimate sure with same explanatory variables
** WFXP 16.11.16
**
** inputs:     Y0 - TxM matriox of dep variables, 
**             X= - matrix of X variablerss without intercept, 
** output:     P - ls estimate  of p matrix of dimensin N x k 
**             sig2_ls unbiased esimate of sig2
*/
proc (6) = sure(Y,X0);
local X, P, Y_hat, U, VC_U, pvec, VC_pv, std_pvec, t_pvec ;
X = ones(rows(X0),1)~X0 ; 
P = Y' X * invpd(X'X);
Y_hat = X * P' ;   
    u = Y - Y_hat;
    VC_U = U'U / (rows(Y)-cols(X)); 
    VC_pv = VC_U .*.  invpd(X'X) ;
    pvec = vec(P');
    std_pvec = sqrt(diag(VC_pv));
    t_pvec = pvec./ std_pvec ;
retp(P,VC_U, VC_pv, pvec, std_pvec, t_pvec);
endp;
@ =========================== END PROC ==================@

@=========================================================@
/*
*  {b_id,lam_bar} = id_simple(pm_mu_m);
** simple procedure to estimate sure with same explanatory variables
** WFXP 16.11.16
**
** inputs:     pm - Pi matrix, 
**             mu_m= - mean market return 
** output:     b_id - identified beta
**            lam_bar - average lambda estimate 
*/
proc (2) = id_simple(pm,mu_m);
local a,b,lam_bar, b_id ;
a = pm[.,1] ;   @ intercept vector@ 
b = pm[.,2] ;   @ slope vector@ 
lam_bar = meanc( (mu_m * b)./(a + mu_m * b));
b_id = b/lam_bar ;
retp(b_id,lam_bar);
endp;
@ =========================== END PROC ==================@


@ ===================== procedures for MDE ============================================= @
/*
** Distance Function with Optimal Weighting Matrix 
*/
proc md(theta);
    local dev;
    dev = pv - fct(theta);
    retp(dev'* invpd(VC_pv) * dev);
endp;

/*
** g function for MD estimation
*/
proc fct(theta);
    local lam_0;
    lam_0 = (1-theta[rows(theta)])*R_m_bar|theta[rows(theta)];
    retp(vec(lam_0 * theta[1:rows(theta)-1]') );
endp;


/*
** Distance Function with Optimal Weighting Matrix under restriction
*/
proc mdr(theta);
    local dev;
    dev = pv - fctr(theta);
    retp(dev'* invpd(VC_pv) * dev);
endp;


/*
** g function for MD estimation under restriction
*/
proc fctr(theta);
    local lam_0;
  @  lam_0 = (1-theta[rows(theta)])*R_m_bar|theta[rows(theta)]; no restriction @
     lam_0 = (1-theta[rows(theta)]^2)*R_m_bar|theta[rows(theta)]^2;  @bounded at zero @
    retp(vec(lam_0 * theta[1:rows(theta)-1]') );
endp;


/*
** Distance Function at some parameter value 
*/
proc md0(theta_0);
    local dev0, lam0;
    lam0 = 0|1 ;
    dev0 = pv - vec(lam0 * theta_0');
    retp(dev0'* invpd(VC_pv) * dev0);
endp;

